/*
 * The internal uOS definitions.
 * Not for the end user.
 *
 * Copyright (C) 2000-2005 Serge Vakulenko, <vak@cronyx.ru>
 *
 * This file is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You can redistribute this file and/or modify it under the terms of the GNU
 * General Public License (GPL) as published by the Free Software Foundation;
 * either version 2 of the License, or (at your discretion) any later version.
 * See the accompanying file "COPYING.txt" for more details.
 *
 * As a special exception to the GPL, permission is granted for additional
 * uses of the text contained in this file.  See the accompanying file
 * "COPY-UOS.txt" for details.
 */
#ifndef __KERNEL_INTERNAL_H_
#define	__KERNEL_INTERNAL_H_ 1

#if __AVR__
#   include <kernel/avr/machdep.h>
#endif
#if defined (__arm__) || defined (__thumb__)
#   include <kernel/arm/machdep.h>
#endif
#if I386
#   include <kernel/i386/machdep.h>
#endif
#if LINUX386
#   include <kernel/linux386/machdep.h>
#endif
#if __MSDOS__
#   include <kernel/i86-dos/machdep.h>
#endif

/*
 * ----------
 * | Task   |
 * | ~~~~   |
 * | lock ----> M
 * |	    |
 * | wait ----> M
 * |	    |
 * | slaves --> M -> M -> M...
 * ---------- <-/----/----/
 */
struct _task_t {
	task_t *	next;		/* double linked list pointers */
	task_t *	prev;
	lock_t *	lock;		/* lock, blocking the task */
	lock_t *	wait;		/* lock, the task is waiting for */
	lock_list_t	slaves;		/* locks, acquired by task */
	void *		message;	/* return value for lock_wait() */
	void *		privatep;	/* private task data pointer */
	char *		name;		/* printable task name */
	int		base_prio;	/* initial task priority */
	int		prio;		/* current task priority */
	MACHDEP_SAVED_STATE_T	stack_context;	/* saved sp when not running */
	lock_t		finish;		/* lock to wain on for task finished */
	char 		stack [1];	/* stack area is placed here */
};

struct _lock_irq_t {
	lock_t *	lock;		/* lock, associated with this irq */
	handler_t	handler;	/* fast interrupt handler */
	void *		arg;		/* argument for fast handler */
	int		irq;		/* irq number */
	char		pending;	/* interrupt is pending */
};

/* The table of interrupt handlers. */
extern lock_irq_t lock_irq [];

/* Current running task. */
extern task_t *task_current;

/* Special `idle' task. */
extern task_t *task_idle;

/* List of tasks ready to run. */
extern task_list_t task_active;

/* Compute the most priority task. */
task_t *task_policy (void);

/* Activate all waiters of the lock. */
void lock_activate (lock_t *m, void *message);

/* Recalculate task priority, based on priorities of acquired locks. */
void task_recalculate_prio (task_t *t);

/* Recalculate lock priority, based on priorities of waiting tasks. */
void lock_recalculate_prio (lock_t *m);

/* Utility functions. */
void task_enqueue (task_list_t *list, task_t *elem);
void task_dequeue (task_list_t *list, task_t *elem);
void lock_enqueue (lock_list_t *list, lock_t *elem);
void lock_dequeue (lock_list_t *list, lock_t *elem);

#define STACK_MAGIC		'\xaa'

#ifndef NDEBUG
#define STACK_GUARD(x)		((x)->stack[0] == STACK_MAGIC)
#endif

#endif /* !__KERNEL_INTERNAL_H_ */
