/*
 * Copyright (C) 2000-2005 Serge Vakulenko, <vak@cronyx.ru>
 *
 * This file is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You can redistribute this file and/or modify it under the terms of the GNU
 * General Public License (GPL) as published by the Free Software Foundation;
 * either version 2 of the License, or (at your discretion) any later version.
 * See the accompanying file "COPYING.txt" for more details.
 *
 * As a special exception to the GPL, permission is granted for additional
 * uses of the text contained in this file.  See the accompanying file
 * "COPY-UOS.txt" for details.
 */
#include <runtime/lib.h>
#include <kernel/uos.h>
#include <kernel/internal.h>

/*
 * Release the lock. All tasks waiting for the lock
 * are unblocked, possibly causing task switch.
 * Disassociate IRQ with the lock, enable interrupts.
 */
void
lock_release_irq (lock_t *m)
{
	int x;

	assert (STACK_GUARD (task_current));
	MACHDEP_INTR_DISABLE (&x);
	assert (m->master != 0);

	/* Remove this lock from the list of task slaves. */
	lock_dequeue (&m->master->slaves, m);

	/* Recalculate the value of task priority.
	 * It must be the maximum of base priority,
	 * and all slave lock priorities. */
	if (m->master->prio <= m->prio && m->master->base_prio < m->prio)
		task_recalculate_prio (m->master);
	m->master = 0;

	/* On pending irq, we must call fast handler. */
	if (m->irq) {
		if (m->irq->pending) {
			m->irq->pending = 0;

			if ((m->irq->handler) (m->irq->arg) == 0) {
				/* Unblock all tasks, waiting for irq. */
				lock_activate (m, 0);
			}
		}
#ifdef MACHDEP_INTR_UNBIND
		MACHDEP_INTR_UNBIND (m->irq->irq);
#endif
		m->irq->lock = 0;
		m->irq = 0;
	}

	if (! m->slaves.head) {
		MACHDEP_INTR_RESTORE (x);
		return;
	}
	do {
		task_t *t = m->slaves.head;
		assert (t->lock == m);
		t->lock = 0;
		task_dequeue (&m->slaves, t);
		task_enqueue (&task_active, t);
	} while (m->slaves.head);
	m->prio = 0;
	MACHDEP_TASK_SWITCH ();
	MACHDEP_INTR_RESTORE (x);
}
