#ifndef __SYS_LIB_H_
#define	__SYS_LIB_H_ 1
#ifdef __cplusplus
extern "C" {
#endif

/*
 * Reading a byte from flash memory.
 * By default - handle it like a simple pointer.
 */
#define FETCH(p) (*(unsigned char*)(p))

/*
 * Creating a constant text string (putting it into a flash memory).
 * By default - handle it like a simple string.
 */
#define CONST(p)	p
#define __flash__	/* void */

#include <runtime/byteorder.h>

#define __LITTLE_ENDIAN 1234
#define __BIG_ENDIAN    4321

#if __AVR__
#   include <stdarg.h>
#   include <runtime/avr/types.h>
#   include <runtime/avr/assert.h>
#   include <runtime/avr/string.h>
#   include <runtime/avr/stdlib.h>
#   include <runtime/avr/io.h>
#   include <runtime/ctype.h>
#   define __BYTE_ORDER __LITTLE_ENDIAN
#endif
#if defined (__arm__) || defined (__thumb__)
#   include <stdarg.h>
#   include <runtime/arm/types.h>
#   include <runtime/arm/assert.h>
#   include <runtime/arm/string.h>
#   include <runtime/arm/stdlib.h>
#   include <runtime/arm/io.h>
#   include <runtime/ctype.h>
#   define __BYTE_ORDER __LITTLE_ENDIAN
#   define __FLOAT_WORD_ORDER __BIG_ENDIAN
#endif
#if I386
#   include <stdarg.h>
#   include <runtime/i386/types.h>
#   include <runtime/i386/assert.h>
#   include <runtime/i386/string.h>
#   include <runtime/i386/stdlib.h>
#   include <runtime/i386/io.h>
#   include <runtime/ctype.h>
#   define __BYTE_ORDER __LITTLE_ENDIAN
#   define __FLOAT_WORD_ORDER __LITTLE_ENDIAN
#endif
#if LINUX386
#   define __timer_t_defined 1
#   include <stdarg.h>
#   include <runtime/linux386/types.h>
#   include <runtime/linux386/assert.h>
#   include <runtime/i386/string.h>
#   include <runtime/ctype.h>
#   define __BYTE_ORDER __LITTLE_ENDIAN
#   define __FLOAT_WORD_ORDER __LITTLE_ENDIAN
#endif
#if __MSDOS__
#   include <runtime/i86-dos/types.h>
#   include <runtime/i86-dos/assert.h>
#   define __BYTE_ORDER __LITTLE_ENDIAN
#   define __FLOAT_WORD_ORDER __LITTLE_ENDIAN
#endif

#ifndef LINUX386
void qsort (void *a, mem_size_t n, mem_size_t es,
	int (*cmp)(const void*, const void*));
void *bsearch (const void *key, const void *base, mem_size_t nmemb, mem_size_t size,
	int (*compar) (const void *, const void *));
char *strstr (const char *haystack, const char *needle);
int strspn (const char *s, const char *accept);
int strcspn (const char *s, const char *reject);
#endif

#ifdef __GNUC__
#   define alloca(size)	__builtin_alloca (size)
#endif

extern int setjmp (jmp_buf);
extern void longjmp (jmp_buf, int);

/*
 * Debugging console interface.
 */
void debug_putchar (void *arg, short c);
unsigned short debug_getchar (void);
int debug_peekchar (void);
void debug_puts (const char *str);
int debug_printf (const char *fmt, ...);
int debug_vprintf (const char *fmt, va_list args);

struct _task_t;
void debug_print_task (struct _task_t *t);

void debug_redirect (void (*func) (void*, short), void *arg);

/*
 * Call global C++ constructors and destructors.
 */
void uos_call_global_initializers (void);
void uos_call_global_destructors (void);

#ifdef __cplusplus
}
#endif
#endif /* !__SYS_LIB_H_ */
